﻿/********************************************************************************/
/* 	gamecore.d																	*/
/*------------------------------------------------------------------------------*/
/*	製作		D.K（あるふぁ～秘密基地）										*/
/*	製作開始	2008/05/22														*/
/*	MAIL		dk@red.interq.or.jp												*/
/*	URL			http://maglog.jp/alpha-secret-base/								*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2008/05/22	初版															*/
/*-その他-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/

private import state;
private import hell2;
private import boot;
private import actor;
private import vector;
private import keypad;
private import clickable;
//private import configparser;

import config;

//private import std.math;
private import map;
private import fmfloader;

/********************************************************************************/

//ConfigParser configparser = null;
const static string CONFIG_PATH = "config.ini";


/* GameMain *********************************************************************/
/*	ゲーム本体の状態															*/
/********************************************************************************/

class GameMain : GameState
{
public:
	//-----------------------------------------------------------------------------
	//システム制御用
	//-----------------------------------------------------------------------------
	enum SCENE{
		INIT = 0,
		START,
		MAIN,
		CLEAR,
		END,
		PAUSE,
		RELESE,
		
		//以下ゲーム内の演出
		PLAYER_WARP,
		ENEMY_GETWAIT,
		PLAYER_MISSWAIT,

		EXIT,
	};
	

	ConfigParser savedata;			// セーブデータ

	int timer;						// 汎用タイマー
	SCENE iSceneStatus;				//画面の状態
	int stage;						// ステージ
	int stage_start_count = 0;		// ステージ開始処理
	int stage_clear_count;			// ステージ開始処理
	int game_score = 0;				//スコア
	int hi_score = 0;				//ハイスコア
	int game_time_count = 0;		//ゲーム終了までの残り時間
	int menu_input = 0;				//敵を倒したときのWAITカウンタ
	int time = 0;

	Keypad keypad;					// キーボード/ゲームパッド取得用

	//枠のサイズ
	const int cx = SCR_W / 2;
	const int cy = SCR_H / 2;

	const int waku_w = 300;
	const int waku_h = 300;
	const int waku_hw = waku_w / 2;
	const int waku_hh = waku_h / 2;

	const int wakuSTx = cx - waku_hw;
	const int wakuSTy = cy - waku_hh;


	//ピースの分割数
	const int piece_x_num = 5;
	const int piece_y_num = 5;
	const int piece_haba_x =  waku_w / piece_x_num;
	const int piece_haba_y =  waku_h / piece_y_num;
	const int piece_haba_hx =  piece_haba_x / 2;
	const int piece_haba_hy =  piece_haba_y / 2;

	int atari_count = 0;

	
	//エフェクト用構造体
	struct EFFECT_DATA
	{
		int exist;	//存在
		int type;	//透明度
		int timer;	//生存時間
		float pos_x;//位置
		float pos_y;//位置
		int r;	//
		int g;	//
		int b;	//
		int alpha;	//透明度
		float dir;	//方向
		float speed;//速度
		float zoom_x;	//
		float zoom_y;	//
		int image_x;
		int image_y;
		int image_w;
		int image_h;
		int anime;
		int anime_max;
		int anime_interinit;
		int anime_inter;
		int param1;
		int param2;
		int param3;
	};
	const int EFFECT_MAX = 128;
	EFFECT_DATA effect[EFFECT_MAX];

	struct PIECE_DATA
	{
		int exist;	//存在
		int type;	//透明度
		int timer;	//生存時間
		float pos_x;//位置
		float pos_y;//位置
		float rot;	//角度
		int r;	//
		int g;	//
		int b;	//
		int alpha;	//透明度
		float dir;	//方向
		float speed;//速度
		int image_x;
		int image_y;
		int rottype;	//角度
		int index;
		int move;	//操作中
		int select;	//
		int pri;	//優先度
		float targetpos_x;//目的位置
		float targetpos_y;//目的位置
		int homing;//ホーミングするか？
		int enable;	//操作可能か？
		float ataripos_x;//目的位置
		float ataripos_y;//目的位置
	};
	const int PIECE_MAX = 128;
	PIECE_DATA piece[PIECE_MAX];
	
	struct POS
	{
		float x;//位置
		float y;//位置
	}
	const int OPTKINSPD_MAX = 9;
	POS kinspdOption[OPTKINSPD_MAX];
	//-----------------------------------------------------------------------------
	//ゲーム別に使用される特別処理用変数
	//-----------------------------------------------------------------------------

	//-----------------------------------------------------------------------------
	//	コンストラクタ
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	this()
	{
		setMsg( GameState.MSG_NONE );
		// 各種初期化
		timer = 0;

		savedata          = new ConfigParser();
		keypad = new Keypad();
		keypad.SetUseCancel( false );

		iSceneStatus = SCENE.INIT;

	}

	
	//-----------------------------------------------------------------------------
	//	シーンの初期化
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void init( )
	{
Hell_write( "Gamecoer init\n" );
		//リソースの読み込み
		Hell_loadTexture("puzzle","resource/image/puzzle_01.bmp" , [0,255,0]);
		Hell_loadTexture("maptip","resource/image/maptip.bmp" , [0,255,0]);
		Hell_loadTexture("bg","resource/image/bg.bmp" , [0,255,0]);
		Hell_loadTexture("msg","resource/image/msg.bmp" , [0,255,0]);
		Hell_loadTexture("effect","resource/image/effect_ball.bmp" , [0,255,0]);
		Hell_loadTexture("menu","resource/image/menu.bmp" , [0,255,0]);


		//セーブデータ
		try
		{
			savedata.load(CONFIG_PATH);
		}
		catch(Exception e)
		{
			savedata.set("HI_SCORE",         999999);
			savedata.write();
		}
		hi_score = savedata.getint("HI_SCORE");

		game_time_count = 0;		//ゲーム終了までの残り時間
		time = 0;

Hell_write( "Gamecoer init\n" );

		iSceneStatus = SCENE.START;
		Hell_stopBgm( 1000 );

		Hell_randSeedTime();

//		create_effect( 3, 320, 280, 0, 0);

		for ( int idx = 0; idx < PIECE_MAX; idx++ )
		{
			piece[idx].exist = false;
		}
		//ピースの分割数
		int i = 0; 
		for ( int x = 0; x < piece_x_num; x++ )
		{
			for ( int y = 0; y < piece_y_num; y++ )
			{
				create_piece( x * piece_haba_x + wakuSTx + piece_haba_hx, y * piece_haba_y + wakuSTy + piece_haba_hy, i, x * piece_haba_x, y * piece_haba_y );
				i++; 
			}
		}

		kinspdOption[0] = POS(256.0,128.0);
		kinspdOption[1] = POS(128.0,64.0);
		kinspdOption[2] = POS(64.0,32.0);
		kinspdOption[3] = POS(32.0,16.0);
		kinspdOption[4] = POS(16.0,8.0);
		kinspdOption[5] = POS(8.0,4.0);
		kinspdOption[6] = POS(4.0,2.0);
		kinspdOption[7] = POS(2.0,1.0);
		kinspdOption[8] = POS(0.0,0.5);
		
		atari_count = 0;
	}
	


	//-----------------------------------------------------------------------------
	//	シーンの後処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void relese( )
	{
		iSceneStatus = SCENE.EXIT;
	}
	//-----------------------------------------------------------------------------
	//	開始演出
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneStart_move( )
	{
		int i;

		stage_start_count++;
		if ( stage_start_count == ( 60 * 3 ) )
		{
			Hell_playBgm( "resource/sound/bgm.ogg", -1, 0 );
			iSceneStatus = SCENE.MAIN;
		}
	}
	//-----------------------------------------------------------------------------
	//	ステージ開始表示処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneStart_draw( )
	{
/*		
		if (stage_start_count < (60 * 4))
		{
			Hell_drawTexture("msg"
				, 0
				, 0
				, 0
				, 0
				, getScreenWidth( ) , getScreenHeight( )
				, 1.0 , 1.0 );
		}
*/
		int mx = 260;
		if ( stage_start_count < ( 60 * 1 ) )
		{
			Hell_drawFont(" 3 " , mx, 180 , 4 , 255,255,255);
		}
		else if ( stage_start_count < ( 60 * 2 ) )
		{
			Hell_drawFont(" 2 " , mx, 180 , 4 , 255,255,255);
		}
		else if ( stage_start_count < ( 60 * 3 ) )
		{
			Hell_drawFont(" 1 " , mx, 180 , 4 , 255,255,255);
		}
		else if ( ( stage_start_count >= ( 60 * 3 ) ) && (stage_start_count < (60 * 4)))
		{
			Hell_drawFont("GO!" , mx, 180 , 4 , 255,255,255);
		}
	}
	//-----------------------------------------------------------------------------
	//	クリア演出の処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneClear_move( )
	{
		if ( stage_clear_count > ( 60 * 2 ) )
		{
			if( Hell_isPushMouse() )
			{
				iSceneStatus = SCENE.RELESE;
			}
		}
		else 
		{
			stage_clear_count++;
		}
		
		
	}
	//-----------------------------------------------------------------------------
	//	クリア演出の表示
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneClear_draw( )
	{
		int min;
		int sec;
		int msec;
		char[] str;

		Hell_setAlpha(HELL_ALPHA_NORMAL);
//		Hell_drawFont( "HI SCORE" , 400, 10 , 2 , 255,255,255);
//		Hell_drawFont( std.string.toString( hi_score ) , 400, 30 , 4 , 255,255,255);

//		Hell_drawFont( "CLEAR!!" , 200, 100 , 4 , 255,255,255);
/*
		min = ( game_score / 60 ) / 60;
		sec = ( game_score / 60 ) % 60;
		msec = game_score % 60;
		str = std.string.format( "%02d:%02d:%02d", min, sec, msec );
*/

		str = std.string.format( "SCOER:%d", game_score );
//		Hell_drawFont( "TIME" ,    210, 300 , 2 , 255,255,255);
//		Hell_drawFont( str , 200, 150 , 4 , 255,255,255);
/*
		if ( hi_score >= game_score )
		{
			Hell_drawFont( "NEW RECORD !",170, 390 , 4 , 255,255,255);
		}
*/
		if ( stage_clear_count > ( 60 * 2 ) )
		{
			Hell_drawFont("MOUSE CLICK TO EXIT" , 150, 340 , 2 , 255,255,255);
		}
		else
		{
			Hell_drawFont( "CLEAR!!" , 200, 100 , 4 , 255,255,255);
		}
	}
	//-----------------------------------------------------------------------------
	//	ゲームオーバー演出
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneEnd( )
	{
//		iSceneStatus = SCENE.MAIN;
	}
	
	//-----------------------------------------------------------------------------
	//	オブジェクトの動作
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void move()
	{
		
		switch( iSceneStatus )
		{
		case SCENE.INIT:
			//初期化
			init( );
			break;
		case SCENE.START:
			//開始
			SeneStart_move( );
			break;
		case SCENE.MAIN:
			keypad.update( );	//リプレイキー入力の更新
			//メイン
			SeneStart_move( );
			piece_move( );
			effect_move( );
			system_move( );

			break;
		case SCENE.CLEAR:
			//クリア
			effect_move( );
			piece_move( );
			SeneClear_move( );
			break;
		case SCENE.END:
			//ゲームオーバー
			break;
		case SCENE.PAUSE:
			//一時停止
			pause_move( );
			break;
		case SCENE.RELESE:
			//後処理
			relese( );
			break;

		case SCENE.EXIT:
		default:
			//シーンの終了
			setMsg(GameState.MSG_REQ_TITLE);
			break;
		}
		
		time++;
	}
	
	//-----------------------------------------------------------------------------
	//	オブジェクトの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void draw()
	{
		// 作画開始
		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		
		switch( iSceneStatus )
		{
		case SCENE.INIT:
			//初期化
			break;
		case SCENE.START:
			//開始
			system_draw( );
			piece_draw( );
			SeneStart_draw( );
			break;
		case SCENE.MAIN:
			//メイン
			system_draw( );
			piece_draw( );
			effect_draw( );
			SeneStart_draw( );
			break;
		case SCENE.CLEAR:
			//クリア
			system_draw( );
			piece_draw( );
			effect_draw( );
			SeneClear_draw( );
			break;
		case SCENE.END:
			//ゲームーバー
			system_draw( );
			piece_draw( );
			SeneEnd( );
			break;
		case SCENE.PAUSE:
			//一時停止
			system_draw( );
			piece_draw( );
			pause_draw( );
			break;
		case SCENE.RELESE:
			//後処理
			break;
			
			
		case SCENE.EXIT:
		default:
			break;
		}
// for debug code start
Hell_drawFPS( 550, 10 );
//Hell_drawFont( std.string.toString( cast( int )scroll_status ), 10, 10 * 1, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )position.y ), 10, 10 * 2, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )place ), 10, 10 * 3, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )motion ), 10, 10 * 4, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )debug_disp[0] ), 10, 10 * 5, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )jump_fream_count ), 10, 10 * 6, 1 , 255,255,255 );
// for debug code end

	}
	
	// 画面サイズ取得
	int getScreenWidth(){return boot.g_width;}
	int getScreenHeight(){return boot.g_height;}
	
	string zfill(string str , int n)
	{
		for(int t=str.length;t<n;t++){
			str = "0" ~ str;
		}
		return str;
	}
	
	//-----------------------------------------------------------------------------
	//	ゲーム情報、システムの処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void system_move( )
	{
		//残り時間のカウント
		game_time_count++;
/*
		if ( Hell_isPushMouse() )
		{
			Hell_playWAV( "shot", 0, -1 );
			int mx = Hell_getMouseX();
			int my = Hell_getMouseY();
			create_effect( 1, mx, my, 0, 0 );

		}
*/
		if ( atari_count == ( piece_x_num * piece_y_num ) )
		{
			
			Hell_playBgm( "resource/sound/results.ogg", 1, 0 );

			//ハイスコアの保存
			game_score = game_time_count;
/*
			if ( hi_score > game_score )
			{
				keypad.save( "replay/replay_01.bin" );	//リプレイの保存
				hi_score = game_score;
				savedata.set("HI_SCORE",   cast(int)hi_score);
				savedata.write();
			}
*/
			iSceneStatus = SCENE.CLEAR;
		}
		
		if ( iSceneStatus == SCENE.MAIN )
		{
			if(keypad.isPushCancel(true))
			{
				//ポーズ
//				iSceneStatus = SCENE.PAUSE;
//				menu_input = 0;
			}
		}
	}
	//-----------------------------------------------------------------------------
	//	ゲーム情報、システムの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void system_draw( )
	{
		int min;
		int sec;
		int msec;
		char[] str;
		int r = 255;
		int g = 255;
		int b = 255;

		Hell_setAlpha(HELL_ALPHA_NORMAL);
		
		
		Hell_drawTextureEx("bg"
			, 640 / 2
			, 360 / 2
			, 0
			, 0
			, 640,360 
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
		
		//枠の表示
		Hell_drawRect( cx - waku_hw, cy - waku_hh, waku_w, waku_h, 0, 0, 0, 0, 0, 128);

	
		if( game_time_count >= ( 60 * 30 ) )
		{
			Hell_drawTextureEx("puzzle"
				, 640 / 2
				, 360 / 2
				, 0
				, 0
				, 300,300 
				, 1.0 , 1.0
				, 0
				,128, 128, 128, 64 );
		}
	
	
		min = ( game_time_count / 60 ) / 60;
		sec = ( game_time_count / 60 ) % 60;
		msec = game_time_count % 60;
		
//		if ( iSceneStatus != SCENE.CLEAR )
		{
			str = std.string.format( "TIME %2d:%02d:%02d", min, sec, msec );
		}
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawFont( str , 200, 10 , 2 , r,g,b);


//		Hell_drawFont( "SCORE" , 10, 10 , 2 , 255,255,255);
/*
		str = std.string.format( "RED TEAM:%d", red_teamscore );
		Hell_drawFont( str , 150, 330 , 3 , 255,255,255);

		str = std.string.format( "SCOER:%d", game_score );
		Hell_drawFont( str , 2, 2 , 2 , 255,255,255);
*/
/*
		min = ( hi_score / 60 ) / 60;
		sec = ( hi_score / 60 ) % 60;
		msec = hi_score % 60;

		Hell_drawFont( "HI SCORE" , 300, 10 , 1 , 255,255,255);
		str = std.string.format( "%02d:%02d:%02d", min, sec, msec );
		Hell_drawFont( str , 300, 30 , 1 , 255,255,255);
*/	
	}
	
	//-----------------------------------------------------------------------------
	//	ポーズの処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void pause_move( )
	{
		if(keypad.isPushUp(true))
		{
			Hell_playWAV( "select", 0, -1 );
			menu_input++;
			if ( menu_input > 1 )
			{
				menu_input = 0;
			}
		}
		else if(keypad.isPushDown(true))
		{
			Hell_playWAV( "select", 0, -1 );
			menu_input--;
			if ( menu_input < 0 )
			{
				menu_input = 1;
			}
		}
		if(keypad.isPushEnter(true))
		{
			switch ( menu_input )
			{
			case 0:
				//ポーズ解除
				iSceneStatus = SCENE.MAIN;
				break;
			case 1:
				//リトライ
				iSceneStatus = SCENE.RELESE;
				break;
			default:
				break;
			}
		}
		if(keypad.isPushCancel(true))
		{
			//ポーズ解除
			iSceneStatus = SCENE.MAIN;
		}
	}
	//-----------------------------------------------------------------------------
	//	ポーズの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void pause_draw( )
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawTexture("menu"
			, 96 + 160
			, 64 + 120 + ( menu_input * 32 )
			, 320
			, 0
			, 32, 32
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
		Hell_drawTexture("menu"
			, 160
			, 120
			, 0
			, 0
			, 320, 240
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
	}
	
	
	//-----------------------------------------------------------------------------
	//	エフェクトの作成
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void create_effect( int type, float x, float y, float dir, float speed, int param1 = 0 )
	{
		int i = 0;
		for ( i = 0; i < EFFECT_MAX; i++ )
		{
			if ( effect[i].exist == 0 )
			{
				effect[i].exist = 1;	//存在
				effect[i].type = type;	//
				effect[i].timer = 0;	//生存時間
				effect[i].pos_x = x;	//位置
				effect[i].pos_y = y;	//位置
				effect[i].alpha = 255;	//透明度
				effect[i].dir = dir;	//
				effect[i].speed = speed;	//
				effect[i].param1 = param1;
				effect[i].param2 = 0;
				
				effect[i].r = 255;	//色
				effect[i].g = 255;	//色
				effect[i].b = 255;	//色
				effect[i].zoom_x = 1.0;	//拡大率
				effect[i].zoom_y = 1.0;	//拡大率
				//タイプによってパラメータを分けるといいかも
				switch (effect[i].type )
				{
				case 0:
					effect[i].r = Hell_randInt(0, 255);	//色
					effect[i].g = Hell_randInt(0, 255);	//色
					effect[i].b = Hell_randInt(0, 255);	//色
					effect[i].zoom_x = 0.5;	//拡大率
					effect[i].zoom_y = 0.5;	//拡大率
					break;
				case 1:
					effect[i].anime = 0;
					effect[i].anime_max = 4;
					effect[i].anime_interinit = 2;
					effect[i].anime_inter = effect[i].anime_interinit;
					break;
				case 2:
					break;
				case 3:
					effect[i].param2 = 10;
					effect[i].param3 = Hell_randInt(0, 10);
					break;
				default:
					break;
				}
				break;
			}
		}
	}
	//エフェクトの表示
	void effect_draw( )
	{
		char[] str;
		int i = 0;
		for ( i = 0; i < EFFECT_MAX; i++ )
		{
			if ( effect[i].exist != 0 )
			{
				Hell_setAlpha( HELL_ALPHA_ADD );	//加算半透明
				switch (effect[i].type )
				{
				case 0:
					Hell_drawTextureEx("effect"
						, effect[i].pos_x
						, effect[i].pos_y
						, 0
						, 0
						, 128, 128
						, effect[i].zoom_x , effect[i].zoom_y
						, 0
						,effect[i].r, effect[i].g, effect[i].b, effect[i].alpha );
					break;
				case 1:
					Hell_drawTextureEx("maptip"
						, effect[i].pos_x
						, effect[i].pos_y
						, 0 + ( effect[i].anime * 64)
						, 160
						, 64, 64
						, effect[i].zoom_x , effect[i].zoom_y
						, 0
						,effect[i].r, effect[i].g, effect[i].b, effect[i].alpha );
					
					break;
				case 2:
					Hell_setAlpha( HELL_ALPHA_NORMAL );	//加算半透明
					if ( effect[i].param1 < 0 )
					{
						str = std.string.format( "%dPTS", effect[i].param1 );
					}
					else
					{
						str = std.string.format( "+%dPTS", effect[i].param1 );
					}
					Hell_drawFont( str , cast(int)effect[i].pos_x, cast(int)effect[i].pos_y , 2 , 255,255,255);
					break;
				case 3:
					Hell_setAlpha( HELL_ALPHA_NORMAL );	//加算半透明
					Hell_drawTexture("maptip"
						, effect[i].pos_x
						, effect[i].pos_y + ( effect[i].param2 * effect[i].param2)
						, 0
						, 256
						, 320 + 64, 96
						, effect[i].zoom_x , effect[i].zoom_y
						, 0
						,effect[i].r, effect[i].g, effect[i].b, effect[i].alpha );
					Hell_drawTexture("msg"
						, effect[i].pos_x
						, effect[i].pos_y + ( effect[i].param2 * effect[i].param2)
						, 0
						, 0 + ( 32 * effect[i].param3 )
						, 512, 32
						, effect[i].zoom_x , effect[i].zoom_y
						, 0
						,effect[i].r, effect[i].g, effect[i].b, effect[i].alpha );
					break;
				default:
					break;
				}
			}
		}
		Hell_setAlpha( HELL_ALPHA_NORMAL );			//通常半透明に戻す
	}
	//エフェクトの移動
	void effect_move( )
	{
		int i = 0;
		for ( i = 0; i < EFFECT_MAX; i++ )
		{
			if ( effect[i].exist != 0 )
			{
				effect[i].timer++;	//生存時間

				float vx = Hell_cosEx( effect[i].dir ) * effect[i].speed;
				float vy = Hell_sinEx( effect[i].dir ) * effect[i].speed;
				effect[i].pos_x += vx;	//位置
				effect[i].pos_y += vy;	//位置
	
				switch (effect[i].type )
				{
				case 0:
					//タイプによって動きを分けるといいかも
					effect[i].zoom_x -= 0.01;	//
					effect[i].zoom_y -= 0.01;	//
					if ( effect[i].zoom_x < 0.0 )
					{
						effect[i].zoom_x = 0.0;
					}
					if ( effect[i].zoom_y < 0.0 )
					{
						effect[i].zoom_y = 0.0;
						effect[i].exist = 0;	//消える 
					}
					break;
				case 1:
					effect[i].anime_inter--;
					if (effect[i].anime_inter <= 0)
					{
						effect[i].anime_inter = effect[i].anime_interinit;
						effect[i].anime++;
						if (effect[i].anime >= effect[i].anime_max)
						{
							effect[i].exist = 0;	//消える 
						}
					}
					
					break;
				case 2:
					if ( effect[i].timer > 60 )
					{
						effect[i].exist = 0;	//消える 
					}
					break;
				case 3:
					effect[i].param2--;
					if ( effect[i].param2 < 0 )
					{
						effect[i].param2 = 0;
					}
					if ( effect[i].timer > 180 )
					{
						effect[i].exist = 0;	//消える 
					}
					break;
				default:
					break;
				}
			}
		}
	}

	//絶対値を求める
	float absf( float val )
	{
		if ( val < 0.0f )
		{
			val = -val;
		}
		return( val );
	}

	//近似値を求める
	float Kinjiti( float x1, float y1, float x2, float y2 )
	{
		float rc = 0;

		float absx = absf( x1 - x2 );
		float absy = absf( y1 - y2 );
	/*	
		if ( absx > absy )
		{
			rc = absy + ( absx / 2.0f );
		}
		else
		{
			rc = absx + ( absy / 2.0f );
		}
	*/

		if ( absx > absy )
		{
			rc = ( absx + absy ) - ( absy / 2.0f );
		}
		else
		{
			rc = ( absx + absy ) - ( absx / 2.0f );
		}
		
		return( rc );
	}
	float TragetKinspeed_Option(POS start, POS end)
	{
		float rc = 0;
		float sa = Kinjiti(start.x, start.y, end.x, end.y);
		int i;
		for (i = 0; i < OPTKINSPD_MAX; i++)
		{
			if (kinspdOption[i].x < absf(sa))
			{
				rc = kinspdOption[i].y;
				if (sa < 0)
				{
					rc = -rc;
				}
				break;
			}
		}

	    return (rc);
	}
	//目的地の角度を取得
	float PointDir( float sx, float sy, float ex, float ey )
	{
		float  rc = 0;

		float px = ex - sx;
		float py = ey - sy;

		rc = Hell_atan2Ex(px, -py);
		
		return( rc );
	}
	//移動後の座標を取得
	void DirMove( float dir, float spd, ref float x, ref float y )
	{
		float move_x;
		float move_y;

		move_x = ( +Hell_sinEx( dir ) * spd );
		move_y = ( -Hell_cosEx( dir ) * spd );

		x = x + move_x;
		y = y + move_y;
		
	}

	//-----------------------------------------------------------------------------
	//	ピースの作成
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void create_piece( int x, int y, int index, int image_x, int image_y )
	{
		int i = 0;
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if ( piece[i].exist == false )
			{
				piece[i].exist = true;	//存在
				piece[i].type = 0;	//透明度
				piece[i].timer = 0;	//生存時間
				piece[i].pos_x = x;//位置
				piece[i].pos_y = y;//位置
				piece[i].rot = 0;	//角度
				piece[i].r = 255;	//
				piece[i].g = 255;	//
				piece[i].b = 255;	//
				piece[i].alpha = 255;	//透明度
				piece[i].dir = 0;	//方向
				piece[i].speed = 0;//速度
				piece[i].image_x = image_x;
				piece[i].image_y = image_y;
				piece[i].rottype = 0;	//角度
				piece[i].index = index;
				piece[i].move = 0;	//操作中
				piece[i].pri = i;	//優先度
				piece[i].select = 0;
				
				float rx = Hell_randInt(piece_haba_hx, wakuSTx - piece_haba_hx );
				float ry = Hell_randInt(piece_haba_hy, SCR_H - piece_haba_hy );
				if ( ( i % 2 ) == 0 )
				{
					rx += wakuSTx + waku_w;
				}
				
				piece[i].targetpos_x = rx;//目的位置
				piece[i].targetpos_y = ry;//目的位置
				piece[i].homing = 1;//ホーミングするか？
				piece[i].enable = 1;	//操作可能か？
				piece[i].ataripos_x = x;//目的位置
				piece[i].ataripos_y = y;//目的位置
//				Hell_write("create_piece x = %d, y = %d, index = %d, image_x = %d, image_y = %d\n", x, y, index, image_x, image_y);
				break;
			}
		}
	}
	void piece_move( )
	{
		//クリック判定
		int mx = Hell_getMouseX();
		int my = Hell_getMouseY();
		int press = false;
		if ( Hell_isPressMouse() )
		{
			press = true;
		}
		atari_count = 0;

		int i = 0;
		//選択
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if ( piece[i].exist == true )
			{
				if( press == false )
				{
					if( piece[i].move == 1 )
					{
						//前回操作していた
						if (
						     ( piece[i].ataripos_x > ( piece[i].pos_x - piece_haba_hx / 2 ) )
						  && ( piece[i].ataripos_x < ( piece[i].pos_x + piece_haba_hx / 2 ) )
						  && ( piece[i].ataripos_y > ( piece[i].pos_y - piece_haba_hy / 2 ) )
						  && ( piece[i].ataripos_y < ( piece[i].pos_y + piece_haba_hy / 2 ) )
						   )
						{
							//正解
							Hell_playWAV( "hit1", 0, -1 );
							piece[i].homing = 1;
							piece[i].targetpos_x = piece[i].ataripos_x;
							piece[i].targetpos_y = piece[i].ataripos_y;
							piece[i].enable = 0;
						}
					}
					piece[i].move = 0;	//移動中やめ
				}
				else
				{
					if( piece[i].move == 1 )
					{
						//移動するピースのばあい
						piece[i].pos_x = mx;
						piece[i].pos_y = my;
					}
				}
				piece[i].select = 0;	//選択表示やめ
				
				if( piece[i].homing != 0 )
				{
					//ホーミングする
					POS spos;
					spos.x = piece[i].pos_x;
					spos.y = piece[i].pos_y;
					POS epos;
					epos.x = piece[i].targetpos_x;
					epos.y = piece[i].targetpos_y;
					piece[i].dir = PointDir( spos.x, spos.y, epos.x, epos.y );
					piece[i].speed = TragetKinspeed_Option(spos, epos );

					DirMove( piece[i].dir, piece[i].speed, piece[i].pos_x, piece[i].pos_y );
					
					if( piece[i].speed < 1 )
					{
						piece[i].homing = 0;
						piece[i].pos_x = piece[i].targetpos_x;
						piece[i].pos_y = piece[i].targetpos_y;
					}

				}
				
				if( piece[i].enable == 0 )
				{
					atari_count++;	//正解数をカウントする
				}
			}
		}
		//移動
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if ( ( piece[i].exist == true ) && ( piece[i].enable == 1 ) )
			{
				if (
				     ( mx > ( piece[i].pos_x - piece_haba_hx) )
				  && ( mx < ( piece[i].pos_x + piece_haba_hx) )
				  && ( my > ( piece[i].pos_y - piece_haba_hy) )
				  && ( my < ( piece[i].pos_y + piece_haba_hy) )
				   )
				{
					if ( Hell_isPushMouse() )
					{
						piece[i].move = 1;	//移動中にする
						Hell_playWAV( "in", 0, -1 );
						create_effect( 1, mx, my, 0, 0 );
						break;
					}
					if( press == false )
					{
						piece[i].select = 1;	//移動中やめ
						break;
					}
				}
			}
		}
	}
	void piece_draw( )
	{
		int i = 0;
		//正解に移動したやつ
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if (( piece[i].exist == true ) && ( piece[i].enable == 0 ))
			{
				Hell_setAlpha(HELL_ALPHA_NORMAL);
				Hell_drawTextureEx("puzzle"
					, piece[i].pos_x
					, piece[i].pos_y
					, piece[i].image_x
					, piece[i].image_y
					, piece_haba_x, piece_haba_y
					, 1 , 1
					, 0
					,piece[i].r, piece[i].g, piece[i].b, piece[i].alpha );
			}
		}
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if ( piece[i].exist == true )
			{
				if ( ( piece[i].enable == 0 ) || ( piece[i].move == 1 ) )
				{
					continue;
				}

				Hell_setAlpha(HELL_ALPHA_NORMAL);
				Hell_drawTextureEx("puzzle"
					, piece[i].pos_x
					, piece[i].pos_y
					, piece[i].image_x
					, piece[i].image_y
					, piece_haba_x, piece_haba_y
					, 1 , 1
					, 0
					,piece[i].r, piece[i].g, piece[i].b, piece[i].alpha );

				if( piece[i].select != 0 )
				{
					Hell_setAlpha(HELL_ALPHA_ADD);
					Hell_drawTextureEx("puzzle"
						, piece[i].pos_x
						, piece[i].pos_y
						, piece[i].image_x
						, piece[i].image_y
						, piece_haba_x, piece_haba_y
						, 1 , 1
						, 0
						,piece[i].r, piece[i].g, piece[i].b, 64 );
				}
				if( piece[i].move != 0 )
				{
					Hell_setAlpha(HELL_ALPHA_ADD);
					Hell_drawTextureEx("puzzle"
						, piece[i].pos_x
						, piece[i].pos_y
						, piece[i].image_x
						, piece[i].image_y
						, piece_haba_x, piece_haba_y
						, 1 , 1
						, 0
						,piece[i].r, piece[i].g, piece[i].b, 64 );
				}
/*
				char[] str;
				str = std.string.format( "%d", piece[i].select );
				Hell_drawFont( str , cast(int)piece[i].pos_x, cast(int)piece[i].pos_y , 1 , 255,255,255);
*/
			}
		}
		for ( i = 0; i < PIECE_MAX; i++ )
		{
			if ( piece[i].exist == true )
			{
				if ( piece[i].move == 1 )
				{
					Hell_setAlpha(HELL_ALPHA_NORMAL);
					Hell_drawTextureEx("puzzle"
						, piece[i].pos_x
						, piece[i].pos_y
						, piece[i].image_x
						, piece[i].image_y
						, piece_haba_x, piece_haba_y
						, 1 , 1
						, 0
						,piece[i].r, piece[i].g, piece[i].b, piece[i].alpha );
					break;
				}
			}
		}
	}

	float getYure( float haba, int time )
	{
		float rc = Hell_cosEx( time ) * haba;
		return(rc);
	}

}



